# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.create_email_template_response_content import CreateEmailTemplateResponseContent
from ..types.email_template_name_enum import EmailTemplateNameEnum
from ..types.get_email_template_response_content import GetEmailTemplateResponseContent
from ..types.set_email_template_response_content import SetEmailTemplateResponseContent
from ..types.update_email_template_response_content import UpdateEmailTemplateResponseContent
from .raw_client import AsyncRawEmailTemplatesClient, RawEmailTemplatesClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class EmailTemplatesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawEmailTemplatesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawEmailTemplatesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawEmailTemplatesClient
        """
        return self._raw_client

    def create(
        self,
        *,
        template: EmailTemplateNameEnum,
        body: typing.Optional[str] = OMIT,
        from_: typing.Optional[str] = OMIT,
        result_url: typing.Optional[str] = OMIT,
        subject: typing.Optional[str] = OMIT,
        syntax: typing.Optional[str] = OMIT,
        url_lifetime_in_seconds: typing.Optional[float] = OMIT,
        include_email_in_redirect: typing.Optional[bool] = OMIT,
        enabled: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateEmailTemplateResponseContent:
        """
        Create an email template.

        Parameters
        ----------
        template : EmailTemplateNameEnum

        body : typing.Optional[str]
            Body of the email template.

        from_ : typing.Optional[str]
            Senders `from` email address.

        result_url : typing.Optional[str]
            URL to redirect the user to after a successful action.

        subject : typing.Optional[str]
            Subject line of the email.

        syntax : typing.Optional[str]
            Syntax of the template body.

        url_lifetime_in_seconds : typing.Optional[float]
            Lifetime in seconds that the link within the email will be valid for.

        include_email_in_redirect : typing.Optional[bool]
            Whether the `reset_email` and `verify_email` templates should include the user's email address as the `email` parameter in the returnUrl (true) or whether no email address should be included in the redirect (false). Defaults to true.

        enabled : typing.Optional[bool]
            Whether the template is enabled (true) or disabled (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateEmailTemplateResponseContent
            Template successfully created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.email_templates.create(
            template="verify_email",
        )
        """
        _response = self._raw_client.create(
            template=template,
            body=body,
            from_=from_,
            result_url=result_url,
            subject=subject,
            syntax=syntax,
            url_lifetime_in_seconds=url_lifetime_in_seconds,
            include_email_in_redirect=include_email_in_redirect,
            enabled=enabled,
            request_options=request_options,
        )
        return _response.data

    def get(
        self, template_name: EmailTemplateNameEnum, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetEmailTemplateResponseContent:
        """
        Retrieve an email template by pre-defined name. These names are `verify_email`, `verify_email_by_code`, `auth_email_by_code`, `reset_email`, `reset_email_by_code`, `welcome_email`, `blocked_account`, `stolen_credentials`, `enrollment_email`, `mfa_oob_code`, `user_invitation`, and `async_approval`. The names `change_password`, and `password_reset` are also supported for legacy scenarios.

        Parameters
        ----------
        template_name : EmailTemplateNameEnum
            Template name. Can be `verify_email`, `verify_email_by_code`, `auth_email_by_code`, `reset_email`, `reset_email_by_code`, `welcome_email`, `blocked_account`, `stolen_credentials`, `enrollment_email`, `mfa_oob_code`, `user_invitation`, `async_approval`, `change_password` (legacy), or `password_reset` (legacy).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetEmailTemplateResponseContent
            Template successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.email_templates.get(
            template_name="verify_email",
        )
        """
        _response = self._raw_client.get(template_name, request_options=request_options)
        return _response.data

    def set(
        self,
        template_name: EmailTemplateNameEnum,
        *,
        template: EmailTemplateNameEnum,
        body: typing.Optional[str] = OMIT,
        from_: typing.Optional[str] = OMIT,
        result_url: typing.Optional[str] = OMIT,
        subject: typing.Optional[str] = OMIT,
        syntax: typing.Optional[str] = OMIT,
        url_lifetime_in_seconds: typing.Optional[float] = OMIT,
        include_email_in_redirect: typing.Optional[bool] = OMIT,
        enabled: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SetEmailTemplateResponseContent:
        """
        Update an email template.

        Parameters
        ----------
        template_name : EmailTemplateNameEnum
            Template name. Can be `verify_email`, `verify_email_by_code`, `auth_email_by_code`, `reset_email`, `reset_email_by_code`, `welcome_email`, `blocked_account`, `stolen_credentials`, `enrollment_email`, `mfa_oob_code`, `user_invitation`, `async_approval`, `change_password` (legacy), or `password_reset` (legacy).

        template : EmailTemplateNameEnum

        body : typing.Optional[str]
            Body of the email template.

        from_ : typing.Optional[str]
            Senders `from` email address.

        result_url : typing.Optional[str]
            URL to redirect the user to after a successful action.

        subject : typing.Optional[str]
            Subject line of the email.

        syntax : typing.Optional[str]
            Syntax of the template body.

        url_lifetime_in_seconds : typing.Optional[float]
            Lifetime in seconds that the link within the email will be valid for.

        include_email_in_redirect : typing.Optional[bool]
            Whether the `reset_email` and `verify_email` templates should include the user's email address as the `email` parameter in the returnUrl (true) or whether no email address should be included in the redirect (false). Defaults to true.

        enabled : typing.Optional[bool]
            Whether the template is enabled (true) or disabled (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SetEmailTemplateResponseContent
            Template successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.email_templates.set(
            template_name="verify_email",
            template="verify_email",
        )
        """
        _response = self._raw_client.set(
            template_name,
            template=template,
            body=body,
            from_=from_,
            result_url=result_url,
            subject=subject,
            syntax=syntax,
            url_lifetime_in_seconds=url_lifetime_in_seconds,
            include_email_in_redirect=include_email_in_redirect,
            enabled=enabled,
            request_options=request_options,
        )
        return _response.data

    def update(
        self,
        template_name: EmailTemplateNameEnum,
        *,
        template: typing.Optional[EmailTemplateNameEnum] = OMIT,
        body: typing.Optional[str] = OMIT,
        from_: typing.Optional[str] = OMIT,
        result_url: typing.Optional[str] = OMIT,
        subject: typing.Optional[str] = OMIT,
        syntax: typing.Optional[str] = OMIT,
        url_lifetime_in_seconds: typing.Optional[float] = OMIT,
        include_email_in_redirect: typing.Optional[bool] = OMIT,
        enabled: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateEmailTemplateResponseContent:
        """
        Modify an email template.

        Parameters
        ----------
        template_name : EmailTemplateNameEnum
            Template name. Can be `verify_email`, `verify_email_by_code`, `auth_email_by_code`, `reset_email`, `reset_email_by_code`, `welcome_email`, `blocked_account`, `stolen_credentials`, `enrollment_email`, `mfa_oob_code`, `user_invitation`, `async_approval`, `change_password` (legacy), or `password_reset` (legacy).

        template : typing.Optional[EmailTemplateNameEnum]

        body : typing.Optional[str]
            Body of the email template.

        from_ : typing.Optional[str]
            Senders `from` email address.

        result_url : typing.Optional[str]
            URL to redirect the user to after a successful action.

        subject : typing.Optional[str]
            Subject line of the email.

        syntax : typing.Optional[str]
            Syntax of the template body.

        url_lifetime_in_seconds : typing.Optional[float]
            Lifetime in seconds that the link within the email will be valid for.

        include_email_in_redirect : typing.Optional[bool]
            Whether the `reset_email` and `verify_email` templates should include the user's email address as the `email` parameter in the returnUrl (true) or whether no email address should be included in the redirect (false). Defaults to true.

        enabled : typing.Optional[bool]
            Whether the template is enabled (true) or disabled (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateEmailTemplateResponseContent
            Template successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.email_templates.update(
            template_name="verify_email",
        )
        """
        _response = self._raw_client.update(
            template_name,
            template=template,
            body=body,
            from_=from_,
            result_url=result_url,
            subject=subject,
            syntax=syntax,
            url_lifetime_in_seconds=url_lifetime_in_seconds,
            include_email_in_redirect=include_email_in_redirect,
            enabled=enabled,
            request_options=request_options,
        )
        return _response.data


class AsyncEmailTemplatesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawEmailTemplatesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawEmailTemplatesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawEmailTemplatesClient
        """
        return self._raw_client

    async def create(
        self,
        *,
        template: EmailTemplateNameEnum,
        body: typing.Optional[str] = OMIT,
        from_: typing.Optional[str] = OMIT,
        result_url: typing.Optional[str] = OMIT,
        subject: typing.Optional[str] = OMIT,
        syntax: typing.Optional[str] = OMIT,
        url_lifetime_in_seconds: typing.Optional[float] = OMIT,
        include_email_in_redirect: typing.Optional[bool] = OMIT,
        enabled: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateEmailTemplateResponseContent:
        """
        Create an email template.

        Parameters
        ----------
        template : EmailTemplateNameEnum

        body : typing.Optional[str]
            Body of the email template.

        from_ : typing.Optional[str]
            Senders `from` email address.

        result_url : typing.Optional[str]
            URL to redirect the user to after a successful action.

        subject : typing.Optional[str]
            Subject line of the email.

        syntax : typing.Optional[str]
            Syntax of the template body.

        url_lifetime_in_seconds : typing.Optional[float]
            Lifetime in seconds that the link within the email will be valid for.

        include_email_in_redirect : typing.Optional[bool]
            Whether the `reset_email` and `verify_email` templates should include the user's email address as the `email` parameter in the returnUrl (true) or whether no email address should be included in the redirect (false). Defaults to true.

        enabled : typing.Optional[bool]
            Whether the template is enabled (true) or disabled (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateEmailTemplateResponseContent
            Template successfully created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.email_templates.create(
                template="verify_email",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            template=template,
            body=body,
            from_=from_,
            result_url=result_url,
            subject=subject,
            syntax=syntax,
            url_lifetime_in_seconds=url_lifetime_in_seconds,
            include_email_in_redirect=include_email_in_redirect,
            enabled=enabled,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self, template_name: EmailTemplateNameEnum, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetEmailTemplateResponseContent:
        """
        Retrieve an email template by pre-defined name. These names are `verify_email`, `verify_email_by_code`, `auth_email_by_code`, `reset_email`, `reset_email_by_code`, `welcome_email`, `blocked_account`, `stolen_credentials`, `enrollment_email`, `mfa_oob_code`, `user_invitation`, and `async_approval`. The names `change_password`, and `password_reset` are also supported for legacy scenarios.

        Parameters
        ----------
        template_name : EmailTemplateNameEnum
            Template name. Can be `verify_email`, `verify_email_by_code`, `auth_email_by_code`, `reset_email`, `reset_email_by_code`, `welcome_email`, `blocked_account`, `stolen_credentials`, `enrollment_email`, `mfa_oob_code`, `user_invitation`, `async_approval`, `change_password` (legacy), or `password_reset` (legacy).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetEmailTemplateResponseContent
            Template successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.email_templates.get(
                template_name="verify_email",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(template_name, request_options=request_options)
        return _response.data

    async def set(
        self,
        template_name: EmailTemplateNameEnum,
        *,
        template: EmailTemplateNameEnum,
        body: typing.Optional[str] = OMIT,
        from_: typing.Optional[str] = OMIT,
        result_url: typing.Optional[str] = OMIT,
        subject: typing.Optional[str] = OMIT,
        syntax: typing.Optional[str] = OMIT,
        url_lifetime_in_seconds: typing.Optional[float] = OMIT,
        include_email_in_redirect: typing.Optional[bool] = OMIT,
        enabled: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SetEmailTemplateResponseContent:
        """
        Update an email template.

        Parameters
        ----------
        template_name : EmailTemplateNameEnum
            Template name. Can be `verify_email`, `verify_email_by_code`, `auth_email_by_code`, `reset_email`, `reset_email_by_code`, `welcome_email`, `blocked_account`, `stolen_credentials`, `enrollment_email`, `mfa_oob_code`, `user_invitation`, `async_approval`, `change_password` (legacy), or `password_reset` (legacy).

        template : EmailTemplateNameEnum

        body : typing.Optional[str]
            Body of the email template.

        from_ : typing.Optional[str]
            Senders `from` email address.

        result_url : typing.Optional[str]
            URL to redirect the user to after a successful action.

        subject : typing.Optional[str]
            Subject line of the email.

        syntax : typing.Optional[str]
            Syntax of the template body.

        url_lifetime_in_seconds : typing.Optional[float]
            Lifetime in seconds that the link within the email will be valid for.

        include_email_in_redirect : typing.Optional[bool]
            Whether the `reset_email` and `verify_email` templates should include the user's email address as the `email` parameter in the returnUrl (true) or whether no email address should be included in the redirect (false). Defaults to true.

        enabled : typing.Optional[bool]
            Whether the template is enabled (true) or disabled (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SetEmailTemplateResponseContent
            Template successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.email_templates.set(
                template_name="verify_email",
                template="verify_email",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.set(
            template_name,
            template=template,
            body=body,
            from_=from_,
            result_url=result_url,
            subject=subject,
            syntax=syntax,
            url_lifetime_in_seconds=url_lifetime_in_seconds,
            include_email_in_redirect=include_email_in_redirect,
            enabled=enabled,
            request_options=request_options,
        )
        return _response.data

    async def update(
        self,
        template_name: EmailTemplateNameEnum,
        *,
        template: typing.Optional[EmailTemplateNameEnum] = OMIT,
        body: typing.Optional[str] = OMIT,
        from_: typing.Optional[str] = OMIT,
        result_url: typing.Optional[str] = OMIT,
        subject: typing.Optional[str] = OMIT,
        syntax: typing.Optional[str] = OMIT,
        url_lifetime_in_seconds: typing.Optional[float] = OMIT,
        include_email_in_redirect: typing.Optional[bool] = OMIT,
        enabled: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateEmailTemplateResponseContent:
        """
        Modify an email template.

        Parameters
        ----------
        template_name : EmailTemplateNameEnum
            Template name. Can be `verify_email`, `verify_email_by_code`, `auth_email_by_code`, `reset_email`, `reset_email_by_code`, `welcome_email`, `blocked_account`, `stolen_credentials`, `enrollment_email`, `mfa_oob_code`, `user_invitation`, `async_approval`, `change_password` (legacy), or `password_reset` (legacy).

        template : typing.Optional[EmailTemplateNameEnum]

        body : typing.Optional[str]
            Body of the email template.

        from_ : typing.Optional[str]
            Senders `from` email address.

        result_url : typing.Optional[str]
            URL to redirect the user to after a successful action.

        subject : typing.Optional[str]
            Subject line of the email.

        syntax : typing.Optional[str]
            Syntax of the template body.

        url_lifetime_in_seconds : typing.Optional[float]
            Lifetime in seconds that the link within the email will be valid for.

        include_email_in_redirect : typing.Optional[bool]
            Whether the `reset_email` and `verify_email` templates should include the user's email address as the `email` parameter in the returnUrl (true) or whether no email address should be included in the redirect (false). Defaults to true.

        enabled : typing.Optional[bool]
            Whether the template is enabled (true) or disabled (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateEmailTemplateResponseContent
            Template successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.email_templates.update(
                template_name="verify_email",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            template_name,
            template=template,
            body=body,
            from_=from_,
            result_url=result_url,
            subject=subject,
            syntax=syntax,
            url_lifetime_in_seconds=url_lifetime_in_seconds,
            include_email_in_redirect=include_email_in_redirect,
            enabled=enabled,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.pagination import AsyncPager, SyncPager
from ...core.request_options import RequestOptions
from ...types.app_metadata import AppMetadata
from ...types.create_organization_invitation_response_content import CreateOrganizationInvitationResponseContent
from ...types.get_organization_invitation_response_content import GetOrganizationInvitationResponseContent
from ...types.list_organization_invitations_offset_paginated_response_content import (
    ListOrganizationInvitationsOffsetPaginatedResponseContent,
)
from ...types.organization_invitation import OrganizationInvitation
from ...types.organization_invitation_invitee import OrganizationInvitationInvitee
from ...types.organization_invitation_inviter import OrganizationInvitationInviter
from ...types.user_metadata import UserMetadata
from .raw_client import AsyncRawInvitationsClient, RawInvitationsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class InvitationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawInvitationsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawInvitationsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawInvitationsClient
        """
        return self._raw_client

    def list(
        self,
        id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        sort: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[OrganizationInvitation, ListOrganizationInvitationsOffsetPaginatedResponseContent]:
        """
        Retrieve a detailed list of invitations sent to users for a specific Organization. The list includes details such as inviter and invitee information, invitation URLs, and dates of creation and expiration. To learn more about Organization invitations, review [Invite Organization Members](https://auth0.com/docs/manage-users/organizations/configure-organizations/invite-members).

        Parameters
        ----------
        id : str
            Organization identifier.

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Defaults to 50.

        include_totals : typing.Optional[bool]
            When true, return results inside an object that also contains the start and limit.  When false (default), a direct array of results is returned.  We do not yet support returning the total invitations count.

        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false). Defaults to true.

        sort : typing.Optional[str]
            Field to sort by. Use field:order where order is 1 for ascending and -1 for descending Defaults to created_at:-1.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[OrganizationInvitation, ListOrganizationInvitationsOffsetPaginatedResponseContent]
            Invitations successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.organizations.invitations.list(
            id="id",
            page=1,
            per_page=1,
            include_totals=True,
            fields="fields",
            include_fields=True,
            sort="sort",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            id,
            page=page,
            per_page=per_page,
            include_totals=include_totals,
            fields=fields,
            include_fields=include_fields,
            sort=sort,
            request_options=request_options,
        )

    def create(
        self,
        id: str,
        *,
        inviter: OrganizationInvitationInviter,
        invitee: OrganizationInvitationInvitee,
        client_id: str,
        connection_id: typing.Optional[str] = OMIT,
        app_metadata: typing.Optional[AppMetadata] = OMIT,
        user_metadata: typing.Optional[UserMetadata] = OMIT,
        ttl_sec: typing.Optional[int] = OMIT,
        roles: typing.Optional[typing.Sequence[str]] = OMIT,
        send_invitation_email: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateOrganizationInvitationResponseContent:
        """
        Create a user invitation for a specific Organization. Upon creation, the listed user receives an email inviting them to join the Organization. To learn more about Organization invitations, review [Invite Organization Members](https://auth0.com/docs/manage-users/organizations/configure-organizations/invite-members).

        Parameters
        ----------
        id : str
            Organization identifier.

        inviter : OrganizationInvitationInviter

        invitee : OrganizationInvitationInvitee

        client_id : str
            Auth0 client ID. Used to resolve the application's login initiation endpoint.

        connection_id : typing.Optional[str]
            The id of the connection to force invitee to authenticate with.

        app_metadata : typing.Optional[AppMetadata]

        user_metadata : typing.Optional[UserMetadata]

        ttl_sec : typing.Optional[int]
            Number of seconds for which the invitation is valid before expiration. If unspecified or set to 0, this value defaults to 604800 seconds (7 days). Max value: 2592000 seconds (30 days).

        roles : typing.Optional[typing.Sequence[str]]
            List of roles IDs to associated with the user.

        send_invitation_email : typing.Optional[bool]
            Whether the user will receive an invitation email (true) or no email (false), true by default

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateOrganizationInvitationResponseContent
            Invitation successfully created.

        Examples
        --------
        from auth0 import (
            Auth0,
            OrganizationInvitationInvitee,
            OrganizationInvitationInviter,
        )

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.organizations.invitations.create(
            id="id",
            inviter=OrganizationInvitationInviter(
                name="name",
            ),
            invitee=OrganizationInvitationInvitee(
                email="email",
            ),
            client_id="client_id",
        )
        """
        _response = self._raw_client.create(
            id,
            inviter=inviter,
            invitee=invitee,
            client_id=client_id,
            connection_id=connection_id,
            app_metadata=app_metadata,
            user_metadata=user_metadata,
            ttl_sec=ttl_sec,
            roles=roles,
            send_invitation_email=send_invitation_email,
            request_options=request_options,
        )
        return _response.data

    def get(
        self,
        id: str,
        invitation_id: str,
        *,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetOrganizationInvitationResponseContent:
        """
        Parameters
        ----------
        id : str
            Organization identifier.

        invitation_id : str
            The id of the user invitation.

        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false). Defaults to true.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetOrganizationInvitationResponseContent
            Invitation successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.organizations.invitations.get(
            id="id",
            invitation_id="invitation_id",
            fields="fields",
            include_fields=True,
        )
        """
        _response = self._raw_client.get(
            id, invitation_id, fields=fields, include_fields=include_fields, request_options=request_options
        )
        return _response.data

    def delete(self, id: str, invitation_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Parameters
        ----------
        id : str
            Organization identifier.

        invitation_id : str
            The id of the user invitation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.organizations.invitations.delete(
            id="id",
            invitation_id="invitation_id",
        )
        """
        _response = self._raw_client.delete(id, invitation_id, request_options=request_options)
        return _response.data


class AsyncInvitationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawInvitationsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawInvitationsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawInvitationsClient
        """
        return self._raw_client

    async def list(
        self,
        id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        sort: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[OrganizationInvitation, ListOrganizationInvitationsOffsetPaginatedResponseContent]:
        """
        Retrieve a detailed list of invitations sent to users for a specific Organization. The list includes details such as inviter and invitee information, invitation URLs, and dates of creation and expiration. To learn more about Organization invitations, review [Invite Organization Members](https://auth0.com/docs/manage-users/organizations/configure-organizations/invite-members).

        Parameters
        ----------
        id : str
            Organization identifier.

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Defaults to 50.

        include_totals : typing.Optional[bool]
            When true, return results inside an object that also contains the start and limit.  When false (default), a direct array of results is returned.  We do not yet support returning the total invitations count.

        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false). Defaults to true.

        sort : typing.Optional[str]
            Field to sort by. Use field:order where order is 1 for ascending and -1 for descending Defaults to created_at:-1.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[OrganizationInvitation, ListOrganizationInvitationsOffsetPaginatedResponseContent]
            Invitations successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.organizations.invitations.list(
                id="id",
                page=1,
                per_page=1,
                include_totals=True,
                fields="fields",
                include_fields=True,
                sort="sort",
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            id,
            page=page,
            per_page=per_page,
            include_totals=include_totals,
            fields=fields,
            include_fields=include_fields,
            sort=sort,
            request_options=request_options,
        )

    async def create(
        self,
        id: str,
        *,
        inviter: OrganizationInvitationInviter,
        invitee: OrganizationInvitationInvitee,
        client_id: str,
        connection_id: typing.Optional[str] = OMIT,
        app_metadata: typing.Optional[AppMetadata] = OMIT,
        user_metadata: typing.Optional[UserMetadata] = OMIT,
        ttl_sec: typing.Optional[int] = OMIT,
        roles: typing.Optional[typing.Sequence[str]] = OMIT,
        send_invitation_email: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateOrganizationInvitationResponseContent:
        """
        Create a user invitation for a specific Organization. Upon creation, the listed user receives an email inviting them to join the Organization. To learn more about Organization invitations, review [Invite Organization Members](https://auth0.com/docs/manage-users/organizations/configure-organizations/invite-members).

        Parameters
        ----------
        id : str
            Organization identifier.

        inviter : OrganizationInvitationInviter

        invitee : OrganizationInvitationInvitee

        client_id : str
            Auth0 client ID. Used to resolve the application's login initiation endpoint.

        connection_id : typing.Optional[str]
            The id of the connection to force invitee to authenticate with.

        app_metadata : typing.Optional[AppMetadata]

        user_metadata : typing.Optional[UserMetadata]

        ttl_sec : typing.Optional[int]
            Number of seconds for which the invitation is valid before expiration. If unspecified or set to 0, this value defaults to 604800 seconds (7 days). Max value: 2592000 seconds (30 days).

        roles : typing.Optional[typing.Sequence[str]]
            List of roles IDs to associated with the user.

        send_invitation_email : typing.Optional[bool]
            Whether the user will receive an invitation email (true) or no email (false), true by default

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateOrganizationInvitationResponseContent
            Invitation successfully created.

        Examples
        --------
        import asyncio

        from auth0 import (
            AsyncAuth0,
            OrganizationInvitationInvitee,
            OrganizationInvitationInviter,
        )

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organizations.invitations.create(
                id="id",
                inviter=OrganizationInvitationInviter(
                    name="name",
                ),
                invitee=OrganizationInvitationInvitee(
                    email="email",
                ),
                client_id="client_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            id,
            inviter=inviter,
            invitee=invitee,
            client_id=client_id,
            connection_id=connection_id,
            app_metadata=app_metadata,
            user_metadata=user_metadata,
            ttl_sec=ttl_sec,
            roles=roles,
            send_invitation_email=send_invitation_email,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self,
        id: str,
        invitation_id: str,
        *,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetOrganizationInvitationResponseContent:
        """
        Parameters
        ----------
        id : str
            Organization identifier.

        invitation_id : str
            The id of the user invitation.

        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false). Defaults to true.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetOrganizationInvitationResponseContent
            Invitation successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organizations.invitations.get(
                id="id",
                invitation_id="invitation_id",
                fields="fields",
                include_fields=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(
            id, invitation_id, fields=fields, include_fields=include_fields, request_options=request_options
        )
        return _response.data

    async def delete(
        self, id: str, invitation_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Parameters
        ----------
        id : str
            Organization identifier.

        invitation_id : str
            The id of the user invitation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organizations.invitations.delete(
                id="id",
                invitation_id="invitation_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, invitation_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .form_field_file_config_category_enum import FormFieldFileConfigCategoryEnum
from .form_field_file_config_storage import FormFieldFileConfigStorage


class FormFieldFileConfig(UniversalBaseModel):
    multiple: typing.Optional[bool] = None
    storage: typing.Optional[FormFieldFileConfigStorage] = None
    categories: typing.Optional[typing.List[FormFieldFileConfigCategoryEnum]] = None
    extensions: typing.Optional[typing.List[str]] = None
    max_size: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="maxSize"), pydantic.Field(alias="maxSize")
    ] = None
    max_files: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="maxFiles"), pydantic.Field(alias="maxFiles")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .flow_action_activecampaign import FlowActionActivecampaign
from .flow_action_airtable import FlowActionAirtable
from .flow_action_auth_0 import FlowActionAuth0
from .flow_action_bigquery import FlowActionBigquery
from .flow_action_clearbit import FlowActionClearbit
from .flow_action_email import FlowActionEmail
from .flow_action_google_sheets import FlowActionGoogleSheets
from .flow_action_http import FlowActionHttp
from .flow_action_hubspot import FlowActionHubspot
from .flow_action_json import FlowActionJson
from .flow_action_jwt import FlowActionJwt
from .flow_action_mailchimp import FlowActionMailchimp
from .flow_action_mailjet import FlowActionMailjet
from .flow_action_otp import FlowActionOtp
from .flow_action_pipedrive import FlowActionPipedrive
from .flow_action_salesforce import FlowActionSalesforce
from .flow_action_sendgrid import FlowActionSendgrid
from .flow_action_slack import FlowActionSlack
from .flow_action_stripe import FlowActionStripe
from .flow_action_telegram import FlowActionTelegram
from .flow_action_twilio import FlowActionTwilio
from .flow_action_whatsapp import FlowActionWhatsapp
from .flow_action_xml import FlowActionXml
from .flow_action_zapier import FlowActionZapier

if typing.TYPE_CHECKING:
    from .flow_action_flow import FlowActionFlow
FlowAction = typing.Union[
    FlowActionActivecampaign,
    FlowActionAirtable,
    FlowActionAuth0,
    FlowActionBigquery,
    FlowActionClearbit,
    FlowActionEmail,
    "FlowActionFlow",
    FlowActionGoogleSheets,
    FlowActionHttp,
    FlowActionHubspot,
    FlowActionJson,
    FlowActionJwt,
    FlowActionMailchimp,
    FlowActionMailjet,
    FlowActionOtp,
    FlowActionPipedrive,
    FlowActionSalesforce,
    FlowActionSendgrid,
    FlowActionSlack,
    FlowActionStripe,
    FlowActionTelegram,
    FlowActionTwilio,
    FlowActionWhatsapp,
    FlowActionXml,
    FlowActionZapier,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .event_stream_cloud_event_group_member_deleted_object_member_1_member_type_enum import (
    EventStreamCloudEventGroupMemberDeletedObjectMember1MemberTypeEnum,
)


class EventStreamCloudEventGroupMemberDeletedObjectMember1(UniversalBaseModel):
    """
    A group member of member_type group
    """

    member_type: EventStreamCloudEventGroupMemberDeletedObjectMember1MemberTypeEnum
    id: str = pydantic.Field()
    """
    The connection member's unique identifier
    """

    type: str = pydantic.Field()
    """
    The type of the connection
    """

    connection_id: str = pydantic.Field()
    """
    Connection ID associated with the member
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.get_job_response_content import GetJobResponseContent
from .raw_client import AsyncRawJobsClient, RawJobsClient

if typing.TYPE_CHECKING:
    from .errors.client import AsyncErrorsClient, ErrorsClient
    from .users_exports.client import AsyncUsersExportsClient, UsersExportsClient
    from .users_imports.client import AsyncUsersImportsClient, UsersImportsClient
    from .verification_email.client import AsyncVerificationEmailClient, VerificationEmailClient


class JobsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawJobsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._users_exports: typing.Optional[UsersExportsClient] = None
        self._users_imports: typing.Optional[UsersImportsClient] = None
        self._verification_email: typing.Optional[VerificationEmailClient] = None
        self._errors: typing.Optional[ErrorsClient] = None

    @property
    def with_raw_response(self) -> RawJobsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawJobsClient
        """
        return self._raw_client

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetJobResponseContent:
        """
        Retrieves a job. Useful to check its status.

        Parameters
        ----------
        id : str
            ID of the job.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetJobResponseContent
            Job retrieved successfully.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.jobs.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    @property
    def users_exports(self):
        if self._users_exports is None:
            from .users_exports.client import UsersExportsClient  # noqa: E402

            self._users_exports = UsersExportsClient(client_wrapper=self._client_wrapper)
        return self._users_exports

    @property
    def users_imports(self):
        if self._users_imports is None:
            from .users_imports.client import UsersImportsClient  # noqa: E402

            self._users_imports = UsersImportsClient(client_wrapper=self._client_wrapper)
        return self._users_imports

    @property
    def verification_email(self):
        if self._verification_email is None:
            from .verification_email.client import VerificationEmailClient  # noqa: E402

            self._verification_email = VerificationEmailClient(client_wrapper=self._client_wrapper)
        return self._verification_email

    @property
    def errors(self):
        if self._errors is None:
            from .errors.client import ErrorsClient  # noqa: E402

            self._errors = ErrorsClient(client_wrapper=self._client_wrapper)
        return self._errors


class AsyncJobsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawJobsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._users_exports: typing.Optional[AsyncUsersExportsClient] = None
        self._users_imports: typing.Optional[AsyncUsersImportsClient] = None
        self._verification_email: typing.Optional[AsyncVerificationEmailClient] = None
        self._errors: typing.Optional[AsyncErrorsClient] = None

    @property
    def with_raw_response(self) -> AsyncRawJobsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawJobsClient
        """
        return self._raw_client

    async def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetJobResponseContent:
        """
        Retrieves a job. Useful to check its status.

        Parameters
        ----------
        id : str
            ID of the job.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetJobResponseContent
            Job retrieved successfully.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.jobs.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    @property
    def users_exports(self):
        if self._users_exports is None:
            from .users_exports.client import AsyncUsersExportsClient  # noqa: E402

            self._users_exports = AsyncUsersExportsClient(client_wrapper=self._client_wrapper)
        return self._users_exports

    @property
    def users_imports(self):
        if self._users_imports is None:
            from .users_imports.client import AsyncUsersImportsClient  # noqa: E402

            self._users_imports = AsyncUsersImportsClient(client_wrapper=self._client_wrapper)
        return self._users_imports

    @property
    def verification_email(self):
        if self._verification_email is None:
            from .verification_email.client import AsyncVerificationEmailClient  # noqa: E402

            self._verification_email = AsyncVerificationEmailClient(client_wrapper=self._client_wrapper)
        return self._verification_email

    @property
    def errors(self):
        if self._errors is None:
            from .errors.client import AsyncErrorsClient  # noqa: E402

            self._errors = AsyncErrorsClient(client_wrapper=self._client_wrapper)
        return self._errors



# This file was auto-generated by Fern from our API Definition.

import typing

from .flow_action_jwt_decode_jwt import FlowActionJwtDecodeJwt
from .flow_action_jwt_sign_jwt import FlowActionJwtSignJwt
from .flow_action_jwt_verify_jwt import FlowActionJwtVerifyJwt

FlowActionJwt = typing.Union[FlowActionJwtDecodeJwt, FlowActionJwtSignJwt, FlowActionJwtVerifyJwt]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from .connection_client_id_salesforce import ConnectionClientIdSalesforce
from .connection_client_secret_salesforce import ConnectionClientSecretSalesforce
from .connection_freeform_scopes_salesforce import ConnectionFreeformScopesSalesforce
from .connection_options_common import ConnectionOptionsCommon
from .connection_scope_salesforce import ConnectionScopeSalesforce
from .connection_set_user_root_attributes_enum import ConnectionSetUserRootAttributesEnum
from .connection_upstream_params import ConnectionUpstreamParams


class ConnectionOptionsSalesforce(ConnectionOptionsCommon):
    """
    Options for the salesforce family of connections (salesforce, salesforce-sandbox, salesforce-community)
    """

    client_id: typing.Optional[ConnectionClientIdSalesforce] = None
    client_secret: typing.Optional[ConnectionClientSecretSalesforce] = None
    freeform_scopes: typing.Optional[ConnectionFreeformScopesSalesforce] = None
    profile: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests the Salesforce profile scope to retrieve basic user information including user_id, organization_id, username, display_name, email, status, photos, and URLs.
    """

    scope: typing.Optional[ConnectionScopeSalesforce] = None
    set_user_root_attributes: typing.Optional[ConnectionSetUserRootAttributesEnum] = None
    upstream_params: typing.Optional[ConnectionUpstreamParams] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class TenantSettingsPasswordPage(UniversalBaseModel):
    """
    Change Password page customization.
    """

    enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether to use the custom change password HTML (true) or the default Auth0 page (false). Default is to use the Auth0 page.
    """

    html: typing.Optional[str] = pydantic.Field(default=None)
    """
    Custom change password HTML (<a href='https://github.com/Shopify/liquid/wiki/Liquid-for-Designers'>Liquid syntax</a> supported).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

EventStreamCloudEventGroupRoleDeletedCloudEventTypeEnum = typing.Union[typing.Literal["group.role.deleted"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

import typing

EncryptionKeyType = typing.Union[
    typing.Literal["customer-provided-root-key", "environment-root-key", "tenant-master-key", "tenant-encryption-key"],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.breached_password_detection_admin_notification_frequency_enum import (
    BreachedPasswordDetectionAdminNotificationFrequencyEnum,
)
from ...types.breached_password_detection_method_enum import BreachedPasswordDetectionMethodEnum
from ...types.breached_password_detection_shields_enum import BreachedPasswordDetectionShieldsEnum
from ...types.breached_password_detection_stage import BreachedPasswordDetectionStage
from ...types.get_breached_password_detection_settings_response_content import (
    GetBreachedPasswordDetectionSettingsResponseContent,
)
from ...types.update_breached_password_detection_settings_response_content import (
    UpdateBreachedPasswordDetectionSettingsResponseContent,
)
from .raw_client import AsyncRawBreachedPasswordDetectionClient, RawBreachedPasswordDetectionClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class BreachedPasswordDetectionClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawBreachedPasswordDetectionClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawBreachedPasswordDetectionClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawBreachedPasswordDetectionClient
        """
        return self._raw_client

    def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetBreachedPasswordDetectionSettingsResponseContent:
        """
        Retrieve details of the Breached Password Detection configuration of your tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBreachedPasswordDetectionSettingsResponseContent
            Breached password detection settings successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.attack_protection.breached_password_detection.get()
        """
        _response = self._raw_client.get(request_options=request_options)
        return _response.data

    def update(
        self,
        *,
        enabled: typing.Optional[bool] = OMIT,
        shields: typing.Optional[typing.Sequence[BreachedPasswordDetectionShieldsEnum]] = OMIT,
        admin_notification_frequency: typing.Optional[
            typing.Sequence[BreachedPasswordDetectionAdminNotificationFrequencyEnum]
        ] = OMIT,
        method: typing.Optional[BreachedPasswordDetectionMethodEnum] = OMIT,
        stage: typing.Optional[BreachedPasswordDetectionStage] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateBreachedPasswordDetectionSettingsResponseContent:
        """
        Update details of the Breached Password Detection configuration of your tenant.

        Parameters
        ----------
        enabled : typing.Optional[bool]
            Whether or not breached password detection is active.

        shields : typing.Optional[typing.Sequence[BreachedPasswordDetectionShieldsEnum]]
            Action to take when a breached password is detected during a login.
                  Possible values: <code>block</code>, <code>user_notification</code>, <code>admin_notification</code>.

        admin_notification_frequency : typing.Optional[typing.Sequence[BreachedPasswordDetectionAdminNotificationFrequencyEnum]]
            When "admin_notification" is enabled, determines how often email notifications are sent.
                    Possible values: <code>immediately</code>, <code>daily</code>, <code>weekly</code>, <code>monthly</code>.

        method : typing.Optional[BreachedPasswordDetectionMethodEnum]

        stage : typing.Optional[BreachedPasswordDetectionStage]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateBreachedPasswordDetectionSettingsResponseContent
            Breached password detection settings successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.attack_protection.breached_password_detection.update()
        """
        _response = self._raw_client.update(
            enabled=enabled,
            shields=shields,
            admin_notification_frequency=admin_notification_frequency,
            method=method,
            stage=stage,
            request_options=request_options,
        )
        return _response.data


class AsyncBreachedPasswordDetectionClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawBreachedPasswordDetectionClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawBreachedPasswordDetectionClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawBreachedPasswordDetectionClient
        """
        return self._raw_client

    async def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetBreachedPasswordDetectionSettingsResponseContent:
        """
        Retrieve details of the Breached Password Detection configuration of your tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBreachedPasswordDetectionSettingsResponseContent
            Breached password detection settings successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.attack_protection.breached_password_detection.get()


        asyncio.run(main())
        """
        _response = await self._raw_client.get(request_options=request_options)
        return _response.data

    async def update(
        self,
        *,
        enabled: typing.Optional[bool] = OMIT,
        shields: typing.Optional[typing.Sequence[BreachedPasswordDetectionShieldsEnum]] = OMIT,
        admin_notification_frequency: typing.Optional[
            typing.Sequence[BreachedPasswordDetectionAdminNotificationFrequencyEnum]
        ] = OMIT,
        method: typing.Optional[BreachedPasswordDetectionMethodEnum] = OMIT,
        stage: typing.Optional[BreachedPasswordDetectionStage] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateBreachedPasswordDetectionSettingsResponseContent:
        """
        Update details of the Breached Password Detection configuration of your tenant.

        Parameters
        ----------
        enabled : typing.Optional[bool]
            Whether or not breached password detection is active.

        shields : typing.Optional[typing.Sequence[BreachedPasswordDetectionShieldsEnum]]
            Action to take when a breached password is detected during a login.
                  Possible values: <code>block</code>, <code>user_notification</code>, <code>admin_notification</code>.

        admin_notification_frequency : typing.Optional[typing.Sequence[BreachedPasswordDetectionAdminNotificationFrequencyEnum]]
            When "admin_notification" is enabled, determines how often email notifications are sent.
                    Possible values: <code>immediately</code>, <code>daily</code>, <code>weekly</code>, <code>monthly</code>.

        method : typing.Optional[BreachedPasswordDetectionMethodEnum]

        stage : typing.Optional[BreachedPasswordDetectionStage]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateBreachedPasswordDetectionSettingsResponseContent
            Breached password detection settings successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.attack_protection.breached_password_detection.update()


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            enabled=enabled,
            shields=shields,
            admin_notification_frequency=admin_notification_frequency,
            method=method,
            stage=stage,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

ConnectionAgentVersionAd = str
"""
Version identifier of the installed AD connector agent software (managed by the AD Connector agent).
"""



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .event_stream_cloud_event_connection_updated_object_6_options_federated_connections_access_tokens import (
    EventStreamCloudEventConnectionUpdatedObject6OptionsFederatedConnectionsAccessTokens,
)
from .event_stream_cloud_event_connection_updated_object_6_options_set_user_root_attributes_enum import (
    EventStreamCloudEventConnectionUpdatedObject6OptionsSetUserRootAttributesEnum,
)
from .event_stream_cloud_event_connection_updated_object_6_options_upstream_params import (
    EventStreamCloudEventConnectionUpdatedObject6OptionsUpstreamParams,
)


class EventStreamCloudEventConnectionUpdatedObject6Options(UniversalBaseModel):
    """
    Options for the 'google-apps' connection
    """

    admin_access_token_expiresin: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Expiration timestamp for the `admin_access_token` in ISO 8601 format. Auth0 uses this value to determine when to refresh the token.
    """

    allow_setting_login_scopes: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When true, allows customization of OAuth scopes requested during user login. Custom scopes are appended to the mandatory email and profile scopes. When false or omitted, only the default email and profile scopes are used. This property is automatically enabled when Token Vault or Connected Accounts features are activated.
    """

    api_enable_groups: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enables integration with the Google Workspace Admin SDK Directory API for groups. When true, Auth0 can synchronize groups & group memberships and supports inbound directory provisioning for groups. Defaults to false.
    """

    api_enable_users: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enables integration with the Google Workspace Admin SDK Directory API. When true, Auth0 can retrieve extended user attributes (admin status, suspension status, group memberships) and supports inbound directory provisioning (SCIM). Defaults to true.
    """

    client_id: str = pydantic.Field()
    """
    Your Google OAuth 2.0 client ID. You can find this in your [Google Cloud Console](https://console.cloud.google.com/apis/credentials) under the OAuth 2.0 Client IDs section.
    """

    domain: typing.Optional[str] = pydantic.Field(default=None)
    """
    Primary Google Workspace domain name that users must belong to.
    """

    domain_aliases: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Email domains associated with this connection for Home Realm Discovery (HRD). When a user's email matches one of these domains, they are automatically routed to this connection during authentication.
    """

    email: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the OAuth flow requests the `email` scope.
    """

    ext_agreed_terms: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Fetches the `agreedToTerms` flag from the Google Directory profile.
    """

    ext_groups: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enables enrichment with Google group memberships (required for `ext_groups_extended`).
    """

    ext_groups_extended: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Controls whether enriched group entries include `id`, `email`, `name` (true) or only the group name (false); can only be set when `ext_groups` is true.
    """

    ext_is_admin: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Fetches the Google Directory admin flag for the signing-in user.
    """

    ext_is_suspended: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Fetches the Google Directory suspended flag for the signing-in user.
    """

    federated_connections_access_tokens: typing.Optional[
        EventStreamCloudEventConnectionUpdatedObject6OptionsFederatedConnectionsAccessTokens
    ] = None
    handle_login_from_social: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, users who sign in with their Google account through a social login will be automatically routed to this Google Workspace connection if their email domain matches the configured tenant_domain or domain_aliases. This ensures enterprise users authenticate through their organization's Google Workspace identity provider rather than through a generic Google social login, enabling access to directory-based attributes and enforcing organizational security policies. Defaults to true for new connections.
    """

    icon_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL for the connection icon displayed in Auth0 login pages. Accepts HTTPS URLs. Used for visual branding in authentication flows.
    """

    map_user_id_to_id: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Determines how Auth0 generates the user_id for Google Workspace users. When false (default), the user's email address is used. When true, Google's stable numeric user ID is used instead, which persists even if the user's email changes. This setting can only be configured when creating the connection and cannot be changed afterward.
    """

    non_persistent_attrs: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    An array of user fields that should not be stored in the Auth0 database (https://auth0.com/docs/security/data-security/denylist)
    """

    profile: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the OAuth flow requests the `profile` scope.
    """

    scope: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Additional OAuth scopes requested beyond the default `email profile` scopes; ignored unless `allow_setting_login_scopes` is true.
    """

    set_user_root_attributes: typing.Optional[
        EventStreamCloudEventConnectionUpdatedObject6OptionsSetUserRootAttributesEnum
    ] = None
    tenant_domain: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Google Workspace primary domain used to identify the organization during authentication.
    """

    upstream_params: typing.Optional[EventStreamCloudEventConnectionUpdatedObject6OptionsUpstreamParams] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionGrantTypesSupported = typing.List[str]
"""
A list of the OAuth 2.0 Grant Type values that this OP supports. Dynamic OpenID Providers MUST support the authorization_code and implicit Grant Type values and MAY support other Grant Types. If omitted, the default value is ["authorization_code", "implicit"].
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .guardian_enrollment_date import GuardianEnrollmentDate
from .guardian_enrollment_status import GuardianEnrollmentStatus


class GetGuardianEnrollmentResponseContent(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    ID for this enrollment.
    """

    status: typing.Optional[GuardianEnrollmentStatus] = None
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Device name (only for push notification).
    """

    identifier: typing.Optional[str] = pydantic.Field(default=None)
    """
    Device identifier. This is usually the phone identifier.
    """

    phone_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    Phone number.
    """

    enrolled_at: typing.Optional[GuardianEnrollmentDate] = None
    last_auth: typing.Optional[GuardianEnrollmentDate] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .branding_colors import BrandingColors
from .branding_font import BrandingFont


class UpdateBrandingResponseContent(UniversalBaseModel):
    colors: typing.Optional[BrandingColors] = None
    favicon_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL for the favicon. Must use HTTPS.
    """

    logo_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL for the logo. Must use HTTPS.
    """

    font: typing.Optional[BrandingFont] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.pagination import AsyncPager, SyncPager
from ...core.request_options import RequestOptions
from ...types.list_user_effective_permissions_response_content import ListUserEffectivePermissionsResponseContent
from ...types.user_effective_permission_response_content import UserEffectivePermissionResponseContent
from .raw_client import AsyncRawEffectivePermissionsClient, RawEffectivePermissionsClient

if typing.TYPE_CHECKING:
    from .sources.client import AsyncSourcesClient, SourcesClient


class EffectivePermissionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawEffectivePermissionsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._sources: typing.Optional[SourcesClient] = None

    @property
    def with_raw_response(self) -> RawEffectivePermissionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawEffectivePermissionsClient
        """
        return self._raw_client

    def list(
        self,
        id: str,
        *,
        resource_server_identifier: str,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[UserEffectivePermissionResponseContent, ListUserEffectivePermissionsResponseContent]:
        """
        Returns the list of effective permissions for a user, taking into account permissions granted directly to the user, as well as those inherited through roles and group memberships.

        Parameters
        ----------
        id : str
            ID of the user to retrieve the permissions for.

        resource_server_identifier : str
            The identifier of the resource server for which to calculate user permissions.

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[UserEffectivePermissionResponseContent, ListUserEffectivePermissionsResponseContent]
            User's effective permissions successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.users.effective_permissions.list(
            id="id",
            from_="from",
            take=1,
            resource_server_identifier="resource_server_identifier",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            id,
            resource_server_identifier=resource_server_identifier,
            from_=from_,
            take=take,
            request_options=request_options,
        )

    @property
    def sources(self):
        if self._sources is None:
            from .sources.client import SourcesClient  # noqa: E402

            self._sources = SourcesClient(client_wrapper=self._client_wrapper)
        return self._sources


class AsyncEffectivePermissionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawEffectivePermissionsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._sources: typing.Optional[AsyncSourcesClient] = None

    @property
    def with_raw_response(self) -> AsyncRawEffectivePermissionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawEffectivePermissionsClient
        """
        return self._raw_client

    async def list(
        self,
        id: str,
        *,
        resource_server_identifier: str,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[UserEffectivePermissionResponseContent, ListUserEffectivePermissionsResponseContent]:
        """
        Returns the list of effective permissions for a user, taking into account permissions granted directly to the user, as well as those inherited through roles and group memberships.

        Parameters
        ----------
        id : str
            ID of the user to retrieve the permissions for.

        resource_server_identifier : str
            The identifier of the resource server for which to calculate user permissions.

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[UserEffectivePermissionResponseContent, ListUserEffectivePermissionsResponseContent]
            User's effective permissions successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.users.effective_permissions.list(
                id="id",
                from_="from",
                take=1,
                resource_server_identifier="resource_server_identifier",
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            id,
            resource_server_identifier=resource_server_identifier,
            from_=from_,
            take=take,
            request_options=request_options,
        )

    @property
    def sources(self):
        if self._sources is None:
            from .sources.client import AsyncSourcesClient  # noqa: E402

            self._sources = AsyncSourcesClient(client_wrapper=self._client_wrapper)
        return self._sources



# This file was auto-generated by Fern from our API Definition.

import typing

SupportedLocales = typing.Union[
    typing.Literal[
        "am",
        "ar",
        "ar-EG",
        "ar-SA",
        "az",
        "bg",
        "bn",
        "bs",
        "ca-ES",
        "cnr",
        "cs",
        "cy",
        "da",
        "de",
        "el",
        "en",
        "en-CA",
        "es",
        "es-419",
        "es-AR",
        "es-MX",
        "et",
        "eu-ES",
        "fa",
        "fi",
        "fr",
        "fr-CA",
        "fr-FR",
        "gl-ES",
        "gu",
        "he",
        "hi",
        "hr",
        "hu",
        "hy",
        "id",
        "is",
        "it",
        "ja",
        "ka",
        "kk",
        "kn",
        "ko",
        "lt",
        "lv",
        "mk",
        "ml",
        "mn",
        "mr",
        "ms",
        "my",
        "nb",
        "nl",
        "nn",
        "no",
        "pa",
        "pl",
        "pt",
        "pt-BR",
        "pt-PT",
        "ro",
        "ru",
        "sk",
        "sl",
        "so",
        "sq",
        "sr",
        "sv",
        "sw",
        "ta",
        "te",
        "th",
        "tl",
        "tr",
        "uk",
        "ur",
        "vi",
        "zgh",
        "zh-CN",
        "zh-HK",
        "zh-MO",
        "zh-TW",
    ],
    typing.Any,
]



from .conftest import get_client, verify_request_count

from auth0.management import ActionTrigger


def test_actions_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "actions.list_.0"
    client = get_client(test_id)
    client.actions.list(
        trigger_id="post-login",
        action_name="actionName",
        deployed=True,
        page=1,
        per_page=1,
        installed=True,
    )
    verify_request_count(
        test_id,
        "GET",
        "/actions/actions",
        {
            "triggerId": "post-login",
            "actionName": "actionName",
            "deployed": "true",
            "page": "1",
            "per_page": "1",
            "installed": "true",
        },
        1,
    )


def test_actions_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "actions.create.0"
    client = get_client(test_id)
    client.actions.create(
        name="name",
        supported_triggers=[
            ActionTrigger(
                id="post-login",
            )
        ],
    )
    verify_request_count(test_id, "POST", "/actions/actions", None, 1)


def test_actions_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "actions.get.0"
    client = get_client(test_id)
    client.actions.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/actions/actions/id", None, 1)


def test_actions_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "actions.delete.0"
    client = get_client(test_id)
    client.actions.delete(
        id="id",
        force=True,
    )
    verify_request_count(test_id, "DELETE", "/actions/actions/id", {"force": "true"}, 1)


def test_actions_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "actions.update.0"
    client = get_client(test_id)
    client.actions.update(
        id="id",
    )
    verify_request_count(test_id, "PATCH", "/actions/actions/id", None, 1)


def test_actions_deploy() -> None:
    """Test deploy endpoint with WireMock"""
    test_id = "actions.deploy.0"
    client = get_client(test_id)
    client.actions.deploy(
        id="id",
    )
    verify_request_count(test_id, "POST", "/actions/actions/id/deploy", None, 1)


def test_actions_test() -> None:
    """Test test endpoint with WireMock"""
    test_id = "actions.test.0"
    client = get_client(test_id)
    client.actions.test(
        id="id",
        payload={"key": "value"},
    )
    verify_request_count(test_id, "POST", "/actions/actions/id/test", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import encode_path_param
from ..core.pagination import AsyncPager, SyncPager
from ..core.parse_error import ParsingError
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.conflict_error import ConflictError
from ..errors.forbidden_error import ForbiddenError
from ..errors.not_found_error import NotFoundError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.connection_name_prefix_template import ConnectionNamePrefixTemplate
from ..types.connection_profile import ConnectionProfile
from ..types.connection_profile_config import ConnectionProfileConfig
from ..types.connection_profile_enabled_features import ConnectionProfileEnabledFeatures
from ..types.connection_profile_name import ConnectionProfileName
from ..types.connection_profile_organization import ConnectionProfileOrganization
from ..types.connection_profile_strategy_overrides import ConnectionProfileStrategyOverrides
from ..types.create_connection_profile_response_content import CreateConnectionProfileResponseContent
from ..types.get_connection_profile_response_content import GetConnectionProfileResponseContent
from ..types.get_connection_profile_template_response_content import GetConnectionProfileTemplateResponseContent
from ..types.list_connection_profile_template_response_content import ListConnectionProfileTemplateResponseContent
from ..types.list_connection_profiles_paginated_response_content import ListConnectionProfilesPaginatedResponseContent
from ..types.update_connection_profile_response_content import UpdateConnectionProfileResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawConnectionProfilesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ConnectionProfile, ListConnectionProfilesPaginatedResponseContent]:
        """
        Retrieve a list of Connection Profiles. This endpoint supports Checkpoint pagination.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 5.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ConnectionProfile, ListConnectionProfilesPaginatedResponseContent]
            Connection Profiles successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "connection-profiles",
            method="GET",
            params={
                "from": from_,
                "take": take,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListConnectionProfilesPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListConnectionProfilesPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.connection_profiles
                _parsed_next = _parsed_response.next
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    from_=_parsed_next,
                    take=take,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        name: ConnectionProfileName,
        organization: typing.Optional[ConnectionProfileOrganization] = OMIT,
        connection_name_prefix_template: typing.Optional[ConnectionNamePrefixTemplate] = OMIT,
        enabled_features: typing.Optional[ConnectionProfileEnabledFeatures] = OMIT,
        connection_config: typing.Optional[ConnectionProfileConfig] = OMIT,
        strategy_overrides: typing.Optional[ConnectionProfileStrategyOverrides] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateConnectionProfileResponseContent]:
        """
        Create a Connection Profile.

        Parameters
        ----------
        name : ConnectionProfileName

        organization : typing.Optional[ConnectionProfileOrganization]

        connection_name_prefix_template : typing.Optional[ConnectionNamePrefixTemplate]

        enabled_features : typing.Optional[ConnectionProfileEnabledFeatures]

        connection_config : typing.Optional[ConnectionProfileConfig]

        strategy_overrides : typing.Optional[ConnectionProfileStrategyOverrides]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateConnectionProfileResponseContent]
            Connection profile successfully created.
        """
        _response = self._client_wrapper.httpx_client.request(
            "connection-profiles",
            method="POST",
            json={
                "name": name,
                "organization": convert_and_respect_annotation_metadata(
                    object_=organization, annotation=ConnectionProfileOrganization, direction="write"
                ),
                "connection_name_prefix_template": connection_name_prefix_template,
                "enabled_features": enabled_features,
                "connection_config": convert_and_respect_annotation_metadata(
                    object_=connection_config, annotation=ConnectionProfileConfig, direction="write"
                ),
                "strategy_overrides": convert_and_respect_annotation_metadata(
                    object_=strategy_overrides, annotation=ConnectionProfileStrategyOverrides, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateConnectionProfileResponseContent,
                    parse_obj_as(
                        type_=CreateConnectionProfileResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_templates(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ListConnectionProfileTemplateResponseContent]:
        """
        Retrieve a list of Connection Profile Templates.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListConnectionProfileTemplateResponseContent]
            Connection Profile Templates successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "connection-profiles/templates",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListConnectionProfileTemplateResponseContent,
                    parse_obj_as(
                        type_=ListConnectionProfileTemplateResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_template(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetConnectionProfileTemplateResponseContent]:
        """
        Retrieve a Connection Profile Template.

        Parameters
        ----------
        id : str
            ID of the connection-profile-template to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetConnectionProfileTemplateResponseContent]
            Connection Profile Template successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"connection-profiles/templates/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetConnectionProfileTemplateResponseContent,
                    parse_obj_as(
                        type_=GetConnectionProfileTemplateResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetConnectionProfileResponseContent]:
        """
        Retrieve details about a single Connection Profile specified by ID.

        Parameters
        ----------
        id : str
            ID of the connection-profile to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetConnectionProfileResponseContent]
            Record for existing connection profile.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"connection-profiles/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetConnectionProfileResponseContent,
                    parse_obj_as(
                        type_=GetConnectionProfileResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Delete a single Connection Profile specified by ID.

        Parameters
        ----------
        id : str
            ID of the connection-profile to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"connection-profiles/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        id: str,
        *,
        name: typing.Optional[ConnectionProfileName] = OMIT,
        organization: typing.Optional[ConnectionProfileOrganization] = OMIT,
        connection_name_prefix_template: typing.Optional[ConnectionNamePrefixTemplate] = OMIT,
        enabled_features: typing.Optional[ConnectionProfileEnabledFeatures] = OMIT,
        connection_config: typing.Optional[ConnectionProfileConfig] = OMIT,
        strategy_overrides: typing.Optional[ConnectionProfileStrategyOverrides] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateConnectionProfileResponseContent]:
        """
        Update the details of a specific Connection Profile.

        Parameters
        ----------
        id : str
            ID of the connection profile to update.

        name : typing.Optional[ConnectionProfileName]

        organization : typing.Optional[ConnectionProfileOrganization]

        connection_name_prefix_template : typing.Optional[ConnectionNamePrefixTemplate]

        enabled_features : typing.Optional[ConnectionProfileEnabledFeatures]

        connection_config : typing.Optional[ConnectionProfileConfig]

        strategy_overrides : typing.Optional[ConnectionProfileStrategyOverrides]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateConnectionProfileResponseContent]
            Connection profile successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"connection-profiles/{encode_path_param(id)}",
            method="PATCH",
            json={
                "name": name,
                "organization": convert_and_respect_annotation_metadata(
                    object_=organization, annotation=ConnectionProfileOrganization, direction="write"
                ),
                "connection_name_prefix_template": connection_name_prefix_template,
                "enabled_features": enabled_features,
                "connection_config": convert_and_respect_annotation_metadata(
                    object_=connection_config, annotation=ConnectionProfileConfig, direction="write"
                ),
                "strategy_overrides": convert_and_respect_annotation_metadata(
                    object_=strategy_overrides, annotation=ConnectionProfileStrategyOverrides, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateConnectionProfileResponseContent,
                    parse_obj_as(
                        type_=UpdateConnectionProfileResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawConnectionProfilesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ConnectionProfile, ListConnectionProfilesPaginatedResponseContent]:
        """
        Retrieve a list of Connection Profiles. This endpoint supports Checkpoint pagination.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 5.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ConnectionProfile, ListConnectionProfilesPaginatedResponseContent]
            Connection Profiles successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "connection-profiles",
            method="GET",
            params={
                "from": from_,
                "take": take,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListConnectionProfilesPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListConnectionProfilesPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.connection_profiles
                _parsed_next = _parsed_response.next
                _has_next = _parsed_next is not None and _parsed_next != ""

                async def _get_next():
                    return await self.list(
                        from_=_parsed_next,
                        take=take,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        name: ConnectionProfileName,
        organization: typing.Optional[ConnectionProfileOrganization] = OMIT,
        connection_name_prefix_template: typing.Optional[ConnectionNamePrefixTemplate] = OMIT,
        enabled_features: typing.Optional[ConnectionProfileEnabledFeatures] = OMIT,
        connection_config: typing.Optional[ConnectionProfileConfig] = OMIT,
        strategy_overrides: typing.Optional[ConnectionProfileStrategyOverrides] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateConnectionProfileResponseContent]:
        """
        Create a Connection Profile.

        Parameters
        ----------
        name : ConnectionProfileName

        organization : typing.Optional[ConnectionProfileOrganization]

        connection_name_prefix_template : typing.Optional[ConnectionNamePrefixTemplate]

        enabled_features : typing.Optional[ConnectionProfileEnabledFeatures]

        connection_config : typing.Optional[ConnectionProfileConfig]

        strategy_overrides : typing.Optional[ConnectionProfileStrategyOverrides]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateConnectionProfileResponseContent]
            Connection profile successfully created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "connection-profiles",
            method="POST",
            json={
                "name": name,
                "organization": convert_and_respect_annotation_metadata(
                    object_=organization, annotation=ConnectionProfileOrganization, direction="write"
                ),
                "connection_name_prefix_template": connection_name_prefix_template,
                "enabled_features": enabled_features,
                "connection_config": convert_and_respect_annotation_metadata(
                    object_=connection_config, annotation=ConnectionProfileConfig, direction="write"
                ),
                "strategy_overrides": convert_and_respect_annotation_metadata(
                    object_=strategy_overrides, annotation=ConnectionProfileStrategyOverrides, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateConnectionProfileResponseContent,
                    parse_obj_as(
                        type_=CreateConnectionProfileResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_templates(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ListConnectionProfileTemplateResponseContent]:
        """
        Retrieve a list of Connection Profile Templates.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListConnectionProfileTemplateResponseContent]
            Connection Profile Templates successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "connection-profiles/templates",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListConnectionProfileTemplateResponseContent,
                    parse_obj_as(
                        type_=ListConnectionProfileTemplateResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_template(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetConnectionProfileTemplateResponseContent]:
        """
        Retrieve a Connection Profile Template.

        Parameters
        ----------
        id : str
            ID of the connection-profile-template to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetConnectionProfileTemplateResponseContent]
            Connection Profile Template successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"connection-profiles/templates/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetConnectionProfileTemplateResponseContent,
                    parse_obj_as(
                        type_=GetConnectionProfileTemplateResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetConnectionProfileResponseContent]:
        """
        Retrieve details about a single Connection Profile specified by ID.

        Parameters
        ----------
        id : str
            ID of the connection-profile to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetConnectionProfileResponseContent]
            Record for existing connection profile.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"connection-profiles/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetConnectionProfileResponseContent,
                    parse_obj_as(
                        type_=GetConnectionProfileResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Delete a single Connection Profile specified by ID.

        Parameters
        ----------
        id : str
            ID of the connection-profile to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"connection-profiles/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        id: str,
        *,
        name: typing.Optional[ConnectionProfileName] = OMIT,
        organization: typing.Optional[ConnectionProfileOrganization] = OMIT,
        connection_name_prefix_template: typing.Optional[ConnectionNamePrefixTemplate] = OMIT,
        enabled_features: typing.Optional[ConnectionProfileEnabledFeatures] = OMIT,
        connection_config: typing.Optional[ConnectionProfileConfig] = OMIT,
        strategy_overrides: typing.Optional[ConnectionProfileStrategyOverrides] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateConnectionProfileResponseContent]:
        """
        Update the details of a specific Connection Profile.

        Parameters
        ----------
        id : str
            ID of the connection profile to update.

        name : typing.Optional[ConnectionProfileName]

        organization : typing.Optional[ConnectionProfileOrganization]

        connection_name_prefix_template : typing.Optional[ConnectionNamePrefixTemplate]

        enabled_features : typing.Optional[ConnectionProfileEnabledFeatures]

        connection_config : typing.Optional[ConnectionProfileConfig]

        strategy_overrides : typing.Optional[ConnectionProfileStrategyOverrides]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateConnectionProfileResponseContent]
            Connection profile successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"connection-profiles/{encode_path_param(id)}",
            method="PATCH",
            json={
                "name": name,
                "organization": convert_and_respect_annotation_metadata(
                    object_=organization, annotation=ConnectionProfileOrganization, direction="write"
                ),
                "connection_name_prefix_template": connection_name_prefix_template,
                "enabled_features": enabled_features,
                "connection_config": convert_and_respect_annotation_metadata(
                    object_=connection_config, annotation=ConnectionProfileConfig, direction="write"
                ),
                "strategy_overrides": convert_and_respect_annotation_metadata(
                    object_=strategy_overrides, annotation=ConnectionProfileStrategyOverrides, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateConnectionProfileResponseContent,
                    parse_obj_as(
                        type_=UpdateConnectionProfileResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

CreateEncryptionKeyType = typing.Union[
    typing.Literal["customer-provided-root-key", "tenant-encryption-key"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

BotDetectionChallengePolicyPasswordlessFlowEnum = typing.Union[
    typing.Literal["never", "when_risky", "always"], typing.Any
]



from __future__ import annotations

import datetime
import uuid
from typing import Any

import jwt


def create_client_assertion_jwt(
    domain: str,
    client_id: str,
    client_assertion_signing_key: str,
    client_assertion_signing_alg: str | None,
) -> str:
    """Creates a JWT for the client_assertion field.

    Args:
        domain (str): The domain of your Auth0 tenant
        client_id (str): Your application's client ID
        client_assertion_signing_key (str): Private key used to sign the client assertion JWT
        client_assertion_signing_alg (str, optional): Algorithm used to sign the client assertion JWT (defaults to 'RS256')

    Returns:
        A JWT signed with the `client_assertion_signing_key`.
    """
    client_assertion_signing_alg = client_assertion_signing_alg or "RS256"
    now = datetime.datetime.utcnow()
    return jwt.encode(
        {
            "iss": client_id,
            "sub": client_id,
            "aud": f"https://{domain}/",
            "iat": now,
            "exp": now + datetime.timedelta(seconds=180),
            "jti": str(uuid.uuid4()),
        },
        client_assertion_signing_key,
        client_assertion_signing_alg,
    )


def add_client_authentication(
    payload: dict[str, Any],
    domain: str,
    client_id: str,
    client_secret: str | None,
    client_assertion_signing_key: str | None,
    client_assertion_signing_alg: str | None,
) -> dict[str, Any]:
    """Adds the client_assertion or client_secret fields to authenticate a payload.

    Args:
        payload (dict): The POST payload that needs additional fields to be authenticated.
        domain (str): The domain of your Auth0 tenant
        client_id (str): Your application's client ID
        client_secret (str, optional): Your application's client secret
        client_assertion_signing_key (str, optional): Private key used to sign the client assertion JWT
        client_assertion_signing_alg (str, optional): Algorithm used to sign the client assertion JWT (defaults to 'RS256')

    Returns:
        A copy of the payload with client authentication fields added.
    """

    authenticated_payload = payload.copy()
    if client_assertion_signing_key:
        authenticated_payload["client_assertion"] = create_client_assertion_jwt(
            domain,
            client_id,
            client_assertion_signing_key,
            client_assertion_signing_alg,
        )
        authenticated_payload[
            "client_assertion_type"
        ] = "urn:ietf:params:oauth:client-assertion-type:jwt-bearer"
    elif client_secret:
        authenticated_payload["client_secret"] = client_secret
    return authenticated_payload



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from .connection_agent_ipad import ConnectionAgentIpad
from .connection_agent_mode_ad import ConnectionAgentModeAd
from .connection_agent_version_ad import ConnectionAgentVersionAd
from .connection_brute_force_protection import ConnectionBruteForceProtection
from .connection_certs_ad import ConnectionCertsAd
from .connection_domain_aliases_ad import ConnectionDomainAliasesAd
from .connection_icon_url import ConnectionIconUrl
from .connection_ips_ad import ConnectionIpsAd
from .connection_options_common import ConnectionOptionsCommon
from .connection_set_user_root_attributes_enum import ConnectionSetUserRootAttributesEnum
from .connection_sign_in_endpoint_ad import ConnectionSignInEndpointAd
from .connection_tenant_domain_ad import ConnectionTenantDomainAd
from .connection_thumbprints_ad import ConnectionThumbprintsAd
from .connection_upstream_params import ConnectionUpstreamParams


class ConnectionOptionsAd(ConnectionOptionsCommon):
    """
    Options for the 'ad' connection
    """

    agent_ip: typing_extensions.Annotated[
        typing.Optional[ConnectionAgentIpad], FieldMetadata(alias="agentIP"), pydantic.Field(alias="agentIP")
    ] = None
    agent_mode: typing_extensions.Annotated[
        typing.Optional[ConnectionAgentModeAd], FieldMetadata(alias="agentMode"), pydantic.Field(alias="agentMode")
    ] = None
    agent_version: typing_extensions.Annotated[
        typing.Optional[ConnectionAgentVersionAd],
        FieldMetadata(alias="agentVersion"),
        pydantic.Field(alias="agentVersion"),
    ] = None
    brute_force_protection: typing.Optional[ConnectionBruteForceProtection] = None
    cert_auth: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="certAuth"),
        pydantic.Field(
            alias="certAuth",
            description="Enables client SSL certificate authentication for the AD connector, requiring HTTPS on the sign-in endpoint",
        ),
    ] = None
    """
    Enables client SSL certificate authentication for the AD connector, requiring HTTPS on the sign-in endpoint
    """

    certs: typing.Optional[ConnectionCertsAd] = None
    disable_cache: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, disables caching of AD connector authentication results to ensure real-time validation against the directory
    """

    disable_self_service_change_password: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, hides the 'Forgot Password' link on login pages to prevent users from initiating self-service password resets
    """

    domain_aliases: typing.Optional[ConnectionDomainAliasesAd] = None
    icon_url: typing.Optional[ConnectionIconUrl] = None
    ips: typing.Optional[ConnectionIpsAd] = None
    kerberos: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enables Windows Integrated Authentication (Kerberos) for seamless SSO when users authenticate from within the corporate network IP ranges
    """

    set_user_root_attributes: typing.Optional[ConnectionSetUserRootAttributesEnum] = None
    sign_in_endpoint: typing_extensions.Annotated[
        typing.Optional[ConnectionSignInEndpointAd],
        FieldMetadata(alias="signInEndpoint"),
        pydantic.Field(alias="signInEndpoint"),
    ] = None
    tenant_domain: typing.Optional[ConnectionTenantDomainAd] = None
    thumbprints: typing.Optional[ConnectionThumbprintsAd] = None
    upstream_params: typing.Optional[ConnectionUpstreamParams] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count


def test_users_effectiveRoles_sources_groups_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "users.effective_roles.sources.groups.list_.0"
    client = get_client(test_id)
    client.users.effective_roles.sources.groups.list(
        id="id",
        role_id="role_id",
        from_="from",
        take=1,
    )
    verify_request_count(
        test_id,
        "GET",
        "/users/id/effective-roles/sources/groups",
        {"role_id": "role_id", "from": "from", "take": "1"},
        1,
    )



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class EventStreamCloudEventOffsetOnlyMessage(UniversalBaseModel):
    """
    An offset-only heartbeat message. Advances the cursor without delivering an event.
    """

    offset: str = pydantic.Field()
    """
    Opaque cursor representing the latest position in the stream. Pass as the `from` query parameter to resume.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .client_create_authentication_method_private_key_jwt_credentials import (
    ClientCreateAuthenticationMethodPrivateKeyJwtCredentials,
)


class ClientCreateAuthenticationMethodPrivateKeyJwt(UniversalBaseModel):
    """
    Defines `private_key_jwt` client authentication method. If this property is defined, the client is enabled to use the Private Key JWT authentication method.
    """

    credentials: ClientCreateAuthenticationMethodPrivateKeyJwtCredentials

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionClaimsLocalesSupported = typing.List[str]
"""
Languages and scripts supported for values in Claims being returned, represented as a JSON array of BCP47 [RFC5646] language tag values. Not all languages and scripts are necessarily supported for all Claim values.
"""



# This file was auto-generated by Fern from our API Definition.

from .connection_userid_attribute_enum_azure_ad import ConnectionUseridAttributeEnumAzureAd

ConnectionUseridAttributeAzureAd = ConnectionUseridAttributeEnumAzureAd
"""
The Azure AD claim to use as the unique user identifier. 'oid' (Object ID) is recommended for single-tenant connections and required for SCIM. 'sub' (Subject) is required for multi-tenant/common endpoint. Only applies with OpenID Connect protocol.
"""



from typing import Any

from .base import AuthenticationBase



class PushedAuthorizationRequests(AuthenticationBase):
    """Pushed Authorization Request (PAR) endpoint"""

    def pushed_authorization_request(
        self, response_type: str, redirect_uri: str, **kwargs
    ) -> Any:
        """Send a Pushed Authorization Request (PAR).

        Args:
             response_type (str): Indicates to Auth0 which OAuth 2.0 flow you want to perform.
             redirect_uri (str): The URL to which Auth0 will redirect the browser after authorization has been granted
             by the user.
             **kwargs: Other fields to send along with the PAR.
             For RAR requests, authorization_details parameter should be added in a proper format. See:https://datatracker.ietf.org/doc/html/rfc9396
             For JAR requests, requests parameter should be send with the JWT as the value. See: https://datatracker.ietf.org/doc/html/rfc9126#name-the-request-request-paramet

        See: https://www.rfc-editor.org/rfc/rfc9126.html
        """
        return self.authenticated_post(
            f"{self.protocol}://{self.domain}/oauth/par",
            data={
                "client_id":self.client_id,
                "client_secret":self.client_secret,
                "response_type": response_type,
                "redirect_uri": redirect_uri,
                **kwargs,
            },
            headers={"Content-Type": "application/x-www-form-urlencoded"},
        )


# This file was auto-generated by Fern from our API Definition.

ConnectionBaseUrlExact = str
"""
Base URL override for the Exact Online API endpoint used for OAuth2 authorization and API requests. Defaults to https://start.exactonline.nl.
"""



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .public_key_credential_algorithm_enum import PublicKeyCredentialAlgorithmEnum
from .public_key_credential_type_enum import PublicKeyCredentialTypeEnum


class PublicKeyCredential(UniversalBaseModel):
    credential_type: PublicKeyCredentialTypeEnum
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Friendly name for a credential.
    """

    pem: str = pydantic.Field()
    """
    PEM-formatted public key (SPKI and PKCS1) or X509 certificate. Must be JSON escaped.
    """

    alg: typing.Optional[PublicKeyCredentialAlgorithmEnum] = None
    parse_expiry_from_cert: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Parse expiry from x509 certificate. If true, attempts to parse the expiry date from the provided PEM. Applies to `public_key` credential type.
    """

    expires_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The ISO 8601 formatted date representing the expiration of the credential. If not specified (not recommended), the credential never expires. Applies to `public_key` credential type.
    """

    kid: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional kid (Key ID), used to uniquely identify the credential. If not specified, a kid value will be auto-generated. The kid header parameter in JWTs sent by your client should match this value. Valid format is [0-9a-zA-Z-_]{10,64}
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.list_action_triggers_response_content import ListActionTriggersResponseContent
from .raw_client import AsyncRawTriggersClient, RawTriggersClient

if typing.TYPE_CHECKING:
    from .bindings.client import AsyncBindingsClient, BindingsClient


class TriggersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTriggersClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._bindings: typing.Optional[BindingsClient] = None

    @property
    def with_raw_response(self) -> RawTriggersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTriggersClient
        """
        return self._raw_client

    def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> ListActionTriggersResponseContent:
        """
        Retrieve the set of triggers currently available within actions. A trigger is an extensibility point to which actions can be bound.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListActionTriggersResponseContent
            The triggers were retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.actions.triggers.list()
        """
        _response = self._raw_client.list(request_options=request_options)
        return _response.data

    @property
    def bindings(self):
        if self._bindings is None:
            from .bindings.client import BindingsClient  # noqa: E402

            self._bindings = BindingsClient(client_wrapper=self._client_wrapper)
        return self._bindings


class AsyncTriggersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTriggersClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._bindings: typing.Optional[AsyncBindingsClient] = None

    @property
    def with_raw_response(self) -> AsyncRawTriggersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTriggersClient
        """
        return self._raw_client

    async def list(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ListActionTriggersResponseContent:
        """
        Retrieve the set of triggers currently available within actions. A trigger is an extensibility point to which actions can be bound.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListActionTriggersResponseContent
            The triggers were retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.actions.triggers.list()


        asyncio.run(main())
        """
        _response = await self._raw_client.list(request_options=request_options)
        return _response.data

    @property
    def bindings(self):
        if self._bindings is None:
            from .bindings.client import AsyncBindingsClient  # noqa: E402

            self._bindings = AsyncBindingsClient(client_wrapper=self._client_wrapper)
        return self._bindings



# This file was auto-generated by Fern from our API Definition.

import typing

from .flow_action_flow_return_json_params_payload_object import FlowActionFlowReturnJsonParamsPayloadObject

FlowActionFlowReturnJsonParamsPayload = typing.Union[FlowActionFlowReturnJsonParamsPayloadObject, str]
